//muliple soundfile convenience
//julian rohrhuber 2000 sa6a014@rrz.uni-hamburg.de
//ben milstein classes included 02/02

//modified version 08/2001 that allows for nil as an entry to keep indexing consistent
//10/2001, allow alloc(index, size)



SignalBank { 
	var  <>signals;
	*new { ^super.new.signals_(Array.new(32)) 
	}
	
	*readAll { arg path, names, startIndex=0;
		this.new.readAll( path, names, startIndex)
	}
	add { arg signal;
	
		var index;
		index = this.getFreeIndex;
		signals = signals.put(index, signal.asArray);
	}
	putAll { arg index, signalArray;
			signalArray = signalArray.asArray;
			if((index + signalArray.size) > (this.size-1), { this.grow(index + 1 - this.size) });
			signalArray.do({ arg item, i;
				signals = signals.put(index + i, item);
			});
	}
	
	put { arg index, signalArray;
			signalArray = signalArray.asArray;
			
			if(signals.size <= index, { 	
				this.grow(index - signals.size + 1)
			});
			signals = signals.put(index, signalArray);
			
	}
	
	readAll { arg path, names, startIndex=0;
		this.grow(names.size + startIndex - this.size);
		names.do({ arg name, i;
			this.read(i + startIndex, path ++ name)
		});
				
	}
	readFileDialog  { 
		GetFileDialog.new({ arg ok, path;
				if (ok, { this.read(nil, path) })
			})
	}
	
	alloc { arg index, size;
		var signal;
		signal = Signal.newClear(size);
		this.put(index, [signal]);
	}
	
	read  { arg index, path;
		var file;
		file = SoundFile.new;
		if(file.read(path), {
			if(index.isNil, { index = this.getFreeIndex });
			if(index > (this.size-1), { this.grow(index + 1 - this.size) });
			signals.put(index, file.data.asArray);
		}, { path.post; " not found.".postln })
	}
	
	readSignal { arg  index, path; 
		var file, signal;
			file = File(path, "r");
			signal = file.readAllSignal;
			file.close;
			this.put(index,signal.asArray)
	}
		
	removeAt { arg index;
		if(index < this.size, { 
			signals.put(index, nil)
		})
	}
	grow { arg growth; 
		signals = signals.growClear(growth);
		
	}	
	getFreeIndex {
		var i;
		i = signals.indexOf(nil);
		if(i.isNil, {this.grow(1); i = this.size - 1; })
		^i
	}
	normalize { signals.do({ arg item; if(item.notNil, {item.do({ arg signal; signal.normalize }) })}) }
	
	size { ^signals.size }
	
	getSize { arg index; 
		var sig; 
		sig = signals.wrapAt(index);
		^if(sig.notNil, {sig.at(0).size}, {0}) 
	}
	
	getSignal { arg index; ^signals.wrapAt(index) }
	getDur { arg index; var dur; dur = this.getSize(index)/this.getSampleRate(index); ^dur }
	getSampleRate {  ^Synth.sampleRate  }
	
}	


SampleBank :  SignalBank {
	var <>paths, <>files;
	
	*new { ^super.new.files_(Array.new(32)).paths_(Array.new(32)) }
	
	
	

	read  { arg index, path;
		var file;
		file = SoundFile.new;
		if(file.read(path), {
			if(index.isNil, { index = this.getFreeIndex });
			if(index > (this.size-1), { this.grow(index + 1 - this.size) });
			signals.put(index, file.data.asArray);
			files.put(index, file);
			paths.put(index, paths);
		}, { path.post; " not found.".postln })
	}
	grow { arg growth; 
		signals = signals.growClear(growth);
		files = files.growClear(growth);
		paths = paths.growClear(growth);
		
	}
	
	
	
	removeAt { arg index;
		if(index < (paths.size-1), { 
		paths.put(index, nil);
		files.put(index, nil);
		signals.put(index, nil)
		})
	}

	getSampleRate { arg index; 
		var file;
		file = files.wrapAt(index);
		^if(file.notNil, { file.sampleRate }, { Synth.sampleRate })
	}
	
}




		
PlayBank { 
	*ar { arg bank, index, playbackRate, offset=0, loopstart=1, loopend=1, mul=1,add=0;
		var signal, size;
		signal = bank.getSignal(index);
		if(signal.isNil, { ^Plug.ar(0) }, {
		size = bank.getSize(index);
		^PlayBuf.ar(signal, bank.getSampleRate(index), playbackRate, offset*size, loopstart*size, loopend*size, mul, add)
		})
		
	}
}

PlayXBank { 
	*ar { arg bank, floatIndex, playbackRate, offset=0, loopstart=1, loopend=1, mul=1,add=0;
		var index;
		index = floatIndex.poll.asInteger;
		^LinXFade2.ar(
			PlayBank.ar(bank, index, playbackRate, offset, loopstart, loopend, mul, add),
			PlayBank.ar(bank, index + 1, playbackRate, offset, loopstart, loopend, mul, add),
			(floatIndex - index) * 2 - 1
		)
	}
}

//from matt rogalsky's TimeStretch class.
//playbackRate from 0.25 to 4 maintains original pitch

PlayBank2 {
	*ar { arg bank, index, playbackRate, offset=0, loopstart=1, loopend=1, mul=1,add=0;
		^PitchShift.ar(
			PlayBank.ar(bank, index, playbackRate, offset=0, loopstart, loopend, mul, add),
			0.1, 					// grain size
			reciprocal(playbackRate),	// shift ratio
			0, 					// pitch dispersion
			0.05					// time dispersion
			)
		}
 
 
 }

//single shot playback (needs no envelope)
BankOsc {
 	*ar{ arg bank, index, dur, mul=1, add=0;
 			var signal;
 			signal = bank.signals.wrapAt(index);
 			^if(signal.notNil, {
 				Osc1.ar(signal, dur, mul, add)
 			}, { nil });
 	
 	}
 	
 	*kr{ arg bank, index, dur, mul=1, add=0;
 			var signal;
 			signal = bank.signals.wrapAt(index);
 			^if(signal.notNil, {
 				Osc1.kr(signal, dur, mul, add)
 			}, { nil });
 	}
 
 }
 
 //ben milstein classes // 2.2001 
 PlayJustLoop2 { 
	
	   *ar { arg bank,index,tempoBPS,nBeats,mul=1.0,add=0;
		var signal,size;
		var rate,loopend,loopstart;
		
		signal = bank.getSignal(index);
		size = bank.getSize(index);

		rate = tempoBPS * (1/nBeats) * (size/44100) ;
			
		^PlayBuf.ar(signal, bank.getSampleRate(index), rate, 0, size-16, size, mul, add)
		
	}
}


 PlayBankLoop3 { 
	
	   *ar { arg bank,index,tempoBPS,nBeats,mChunks,offset=0,mul=1.0,add=0;
			var signal,size;
			var rate,loopend,loopstart,env;
		
			signal = bank.getSignal(index);
			size = bank.getSize(index);
	
			rate = tempoBPS * (1/nBeats) * (size/44100) ;
			
			loopstart =  offset * size/mChunks;
 			
 			loopend = (size/mChunks) + loopstart;
 			 		
		^PlayBuf.ar(signal, bank.getSampleRate(index), rate, loopstart, loopstart, loopend, mul, add)
		
	}
}
